import pg from "pg";
import { ClassKind } from "../types/enums/ClassKind.js";
import { r } from "../util/chalk.js";
import { LOGGER } from "../util/constants.js";
import { getAttributes } from "../util/functions/getters/getAttributes.js";
import { getClasses } from "../util/functions/getters/getClasses.js";
import { getConsoleHeader } from "../util/functions/getters/getConsoleHeader.js";
import { getConstraints } from "../util/functions/getters/getConstraints.js";
import { getDataTypes } from "../util/functions/getters/getDataTypes.js";
import { getTables } from "../util/functions/getters/getTables.js";
export class Fetcher {
    constructor(config, progressBar, connection, generatorConfig) {
        this.config = config;
        this.progressBar = progressBar;
        this.connection = connection;
        this.generatorConfig = generatorConfig;
        this.dbName = "";
        this.tables = [];
        this.dataTypes = [];
        this.classes = [];
        this.attributes = [];
        this.constraints = [];
        this.LOGGER = LOGGER?.extend("Fetcher");
        this.client = new pg.Client({
            connectionString: this.formatted_connection_string,
            ssl: { rejectUnauthorized: false },
        });
    }
    async connect() {
        try {
            await this.client.connect();
            await this.setDatabaseName();
        }
        catch {
            this.progressBar.stop();
            if (this.generatorConfig?.noConsoleLogs !== true) {
                console.log(getConsoleHeader(r("Could not connect to database!"), "Please check your connection settings.", false, `Used connection string: ${this.formatted_connection_string}`));
            }
            if (this.generatorConfig?.throwOnError === true)
                throw new Error(`Could not connect to database! Used connection string: ${this.formatted_connection_string}`);
            process.exit(1);
        }
    }
    async setDatabaseName() {
        const { rows: [{ current_database: currentDatabase }], } = await this.client.query(`
			SELECT current_database();
		`);
        this.dbName = currentDatabase;
        this.LOGGER = LOGGER?.extend("Fetcher").extend(currentDatabase);
        this.LOGGER?.("Connected to database: %s", currentDatabase);
    }
    get formatted_connection_string() {
        if (typeof this.connection === "string")
            return this.connection;
        return this.connection.password
            ? `postgres://${this.connection.user}:${this.connection.password}@${this.connection.host}:${this.connection.port}/${this.connection.database}`
            : `postgres://${this.connection.user}@${this.connection.host}:${this.connection.port}/${this.connection.database}`;
    }
    get schema_names() {
        return [...new Set(this.tables.map(({ schema_name: schemaName }) => schemaName))];
    }
    async fetchTables() {
        this.LOGGER?.("Fetching tables...");
        const tables = await getTables(this.client, this.config, this.dbName);
        this.LOGGER?.("Fetched %d tables", tables.length);
        if (tables.length === 0) {
            this.progressBar.stop();
            if (this.generatorConfig?.noConsoleLogs !== true) {
                console.log(getConsoleHeader(r("Could not fetch any tables!"), "Please check your schemas/tables settings.", false, `Used connection string: ${this.formatted_connection_string}`));
            }
            if (this.generatorConfig?.throwOnError === true)
                throw new Error("Could not fetch any tables!");
            process.exit(1);
        }
        this.tables = tables;
        this.progressBar.incrementProgress();
    }
    async fetchDataTypes() {
        this.LOGGER?.("Fetching data types...");
        const types = await getDataTypes(this.client, this.schema_names);
        this.LOGGER?.("Fetched %d data types", types.length);
        if (types.length === 0) {
            this.progressBar.stop();
            if (this.generatorConfig?.noConsoleLogs !== true) {
                console.log(getConsoleHeader(r("Could not fetch any data types!"), "Please check your schemas/tables settings.", false, `Used connection string: ${this.formatted_connection_string}`));
            }
            if (this.generatorConfig?.throwOnError === true)
                throw new Error("Could not fetch any data types!");
            process.exit(1);
        }
        this.dataTypes = types;
        this.progressBar.incrementProgress();
    }
    async fetchClasses() {
        this.LOGGER?.("Fetching classes...");
        const classes = await getClasses(this.client, {
            schema_names: this.schema_names,
            kind: [ClassKind.OrdinaryTable, ClassKind.View, ClassKind.MaterializedView],
        });
        this.LOGGER?.("Fetched %d classes", classes.length);
        if (classes.length === 0) {
            this.progressBar.stop();
            if (this.generatorConfig?.noConsoleLogs !== true) {
                console.log(getConsoleHeader(r("Could not fetch any classes!"), "Please check your schemas/tables settings.", false, `Used connection string: ${this.formatted_connection_string}`));
            }
            if (this.generatorConfig?.throwOnError === true)
                throw new Error("Could not fetch any classes!");
            process.exit(1);
        }
        this.classes = classes;
        this.progressBar.incrementProgress();
    }
    async fetchAttributes() {
        this.LOGGER?.("Fetching attributes...");
        const attributes = await getAttributes(this.client, {
            schema_names: this.schema_names,
            database_name: this.dbName,
        });
        this.LOGGER?.("Fetched %d attributes", attributes.length);
        if (attributes.length === 0) {
            this.progressBar.stop();
            if (this.generatorConfig?.noConsoleLogs !== true) {
                console.log(getConsoleHeader(r("Could not fetch any attributes!"), "Please check your schemas/tables settings.", false, `Used connection string: ${this.formatted_connection_string}`));
            }
            if (this.generatorConfig?.throwOnError === true)
                throw new Error("Could not fetch any attributes!");
            process.exit(1);
        }
        this.attributes = attributes;
        this.progressBar.incrementProgress();
    }
    async fetchConstraints() {
        this.LOGGER?.("Fetching constraints...");
        const constraints = await getConstraints(this.client, this.schema_names);
        this.LOGGER?.("Fetched %d constraints", constraints.length);
        this.constraints = constraints;
        this.progressBar.incrementProgress();
    }
    get fetchedData() {
        return {
            database: this.dbName,
            tables: this.tables,
            types: this.dataTypes,
            classes: this.classes.map((cls) => ({
                ...cls,
                attributes: this.attributes.filter(att => att.class_id === cls.class_id),
                constraints: this.constraints.filter(con => con.class_id === cls.class_id),
            })),
        };
    }
    async disconnect() {
        this.LOGGER?.("Disconnecting from database...");
        await this.client.end();
        this.LOGGER?.("Disconnected from database");
    }
}
//# sourceMappingURL=Fetcher.js.map